# Changelog

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).

## [2.2.0] - 2025-03-13

### Added

- Extended PDF Signature verification that implements detection and classification modifications
  - GUI / Report: add new Annotations for extended PDF signature verification results
  - WS: add new Annotations and Codes for extended PDF signature verification results 

### Changed

- Requires Java 17 and Tomcat 10
- GUI: custom icon for certificate-validation result and qualified certificates
- PDF Report: custom label for certificate-validation result and qualified certificates
- Update third-party libs

### Upgrade Config From 2.1.* to 2.2.0

Pruefdienst 2.2.0 requires an update runtime environment:

- Java 17 (openJDK, Oracle JDK, .. )
- Apache Tomcat 10

MOA-SPSS 3.2.x still requires:

- Java 11 (openJDK, Oracle JDK, .. )
- Apache Tomcat 9

Environment updates:

- Download and install a current version of Java 17 
- Download and install a current version of Apache Tomcat 10
- Copy / transfer the current Pruefdienst configuration to new Tomcat / Java environment

Configuration updates

- Changes in PDF report requires an update of report generation template.
  Replace the following file:

  `conf/signature-verification/protocolgeneration/templates/report_fop_stylesheet.xsl`
  by using the update version from release package

## [2.1.12] - 2024-09-04

### Fixed

- Optimize verification of invalid signed PDF documents where the signature byte range does not match the document structure

## [2.1.11] - 2024-09-03

### Fixed

- Document detection in case of documents based on PDF 2.0 specification

### General

- Update third-party libs

## [2.1.10] - 2023-09-13

### Fixed

- Optimize  PDF-AS 3.x validation in case of time-intensive text-extraction
- Add missing byte-ranges in case of PDF signatures with PKCS#1 Signature-Filter

## [2.1.9] - 2023-08-02

### Fixed

- Language selection by HTTP GET parameter
- Language validation in SOAP WebService
- Optimize annotation of verification report in case of error's during PDF-AS 3.x validation
- Optimized logging

## [2.1.8] - 2023-07-24

### Fixed

- Hotfix for error handling in case pdf-as signature verification fails

## [2.1.7] - 2023-06-30

### Fixed

- GUI: missing data-type suffix on downloaded files

## [2.1.6] - 2023-04-13

### Fixed

- GUI: Relax validation of filename when uploading a document.
  Only prohibit the following characters: `<>:"/\|?*`and Unicode from`\u0000-\u001F`

## [2.1.5] - 2023-03-31

### Added

- CONFIG: Set missing configuration properties to allow file sizes bigger than 10MB

### Fixed

- GUI: Insufficient verification result information in case of PDF documents with valid signature that has a incremental update
  - Optimize UX information
  - Add verification level _orange_ into PDF based verification report
- Core: Optimize error-handling and error messages
- LOG: optimize logging

## [2.1.4] - 2023-03-27

### Added

- CONFIG: Add configuration to set external Web-Service endpoint URL
- LOG: add transactionID as log parameter

### Fixed

- CVE-2023-20860, CVE-2023-20861
- LOG: optimize logging

## [2.1.3] - 2023-03-16

### Fixed

- GUI: Relax validation of filename when uploading a document.
  Allow the following characters: `()[]{}%&=#`.
  Prohibit the following characters: `,`

## [2.1.2] - 2023-02-28

internal version only

## [2.1.1] - 2023-01-24

### Added

- CONFIG: Add default logback config with file appender and time based
  rollover / trigger policy.
- CONFIG: Add path to default logback config (disabled by default).
- DOC: Update Pruefdienst Documentation.
- DOC: Update Pruefdienst Webservice Documentation.
- DOC: Add I18n Guide.
- RELEASE: Add copy of i18n resources.
- RELEASE: Add copy of web service schemata.

### Fixed

- GUI: Dont escape html tags in upload form's error messages.
- GUI: Drop speculative hint about document compression causing the
  "This document cannot be processed" error thrown by PDFAS (because
  its not always the cause).
- GUI: Fix Problem with accept header locales and i18n.
- CONFIG: Fix PdfAs / Binary Sig. "sigType 'null'" exception.

### Removed

- GUI: Remove outdated hrefs in i18n labels.
- LOGGING: Remove noisy logging statement in
  AbstractStrafRegBeschFormatDetector.

### Upgrade Guide from 2.1.0

- No actions required - all changes are backwards compatible.
- If you want to use a production ready logback configuration, copy
  logback.xml into `$CATALINA_BASE/conf/signature-verification/` and
  enable it by symlinking to logback.xml like so:
  
  ```
  cd conf && ln -s signature-verification/logback.xml
  ```
- Fix inability of PdfAs to verify binary signatures by adding a default profile to
  `conf/signature-verification/pdf-as/cfg/config.properties`:
  
  ```
  # PROFIL: Standardsignatur Deutsch, PDF/A-1b
  sig_obj.type.default=SIGNATURBLOCK_DE
  ```

## [2.1.0] - 2023-01-13

### Added

- GUI: Add breadcrumb menu and allow user to navigate back and forth
  between result and upload form.
- GUI: Add *verdict* row in overview section of result view. This
  verdict describes if signature is valid or if app found problems in
  the verification (and which problems). The background color of the
  verdict row matches the verdict (green: success, yellow: warning,
  red: error).
- WS: Enable XML schema validation for incoming requests.
- CONFIG: Add `conf/signature-verification/frontend-app/pruefdienst.yaml`.

### Changed

- GUI: Adapt Look & Feel to [RTR's Web Formular](https://www.rtr.at/webformular)
- GUI: Combine result's overview page and detail page in a single page.
  Result details are hidden behind collapsible panels.
- GUI: Move signature annotations into detail panels.
- GUI: Replace acronyms for "Signature", "Manifest", and "Certificate"
  with Font-Awesome icons.
- WS: Change resource locations for XSD files. The new location
  requires a URL parameter with the filename of the schema file, E.g.:
  `?xsd=verificationservice.xsd`.
- WS: Change namespace prefix of the SOAP Namespace to `soap` (was `soapenv`).
- WS: Change error code in `Envelope/Body/Fault/Code` element upon a
  schema validation error to `Sender` (was `Receiver`).
- CONFIG: Change i18n message overriding mechanism:
  Override messages via `conf/signature-verification/frontend-app/i18n/labels`.
- CONFIG: Change the following default values (= values that the app
  falls back to in case there is no value assigned to it in the
  external configuration file `conf/signature-verification/application_config.xml`):

  | Property                             | New Default Value                                                                      |
  |:-------------------------------------|:---------------------------------------------------------------------------------------|
  | `general/detached.signature.enabled` | `true`                                                                                 |
  | `general/form.validation.enabled`    | `true`                                                                                 |
  | `pdf-as/resource.path`               | `./conf/signature-verification/pdf-as`                                                 |
  | `adobe/support.all.filters`          | `true`                                                                                 |
  | `adobe/supported.subfilters`         | Add filters[^1]                                                                        |
  | `srb/resource.path`                  | `./conf/signature-verification/srb`                                                    |
  | `moa/sp/connection.url`              | `http://localhost:7087/moa-spss/services/SignatureVerification`                        |
  | `moa/sp/trustprofile.id`             | `signature-verification-trustprofile-id`                                               |
  | `moa/ss/connection.url`              | `http://localhost:7087/moa-spss/services/SignatureCreation`                            |
  | `moa/ss/key.idendifier`              | `signature-verification-keygroup-id`                                                   |
  | `report/pdfreport.enabled`           | `true`                                                                                 |
  | `templates/report/stylesheet.fop`    | `./conf/signature-verification/protocolgeneration/templates/report_fop_stylesheet.xsl` |
  | `format-to-verifyclient`             | Add links for all recognized formats;                                                  |
  |                                      | Update class references to match new package structure[^1]                             |
  | `format-filter`                      | Add `officialsignature` and `certificate_annotation` category[^1]                      |

- CONFIG: Update default configuration for Format Detector
  (`formatdetection-config.xml`). Update classes to reflect updated
  package structure. Remove outdated detectors.
- CONFIG: Change how to show RTR disclaimer at the bottom of GUI and
  in PDF verification report. Before: Show RTR disclaimer when app is
  compiled with a specific profile. Now: Show RTR disclaimer per
  default, disable it by removing the value of the
  `verification.result.text.overview.disclaimer.text` in
  `conf/signature-verification/frontend-app/i18n/labels`.

### Fixed

- WS: Fix annotation code of certain signature annotations in
  `VerificationReport/SignatureInfo/Annotations`. Before, the service
  returned `-1` for certain annotations. Now, the service returns the
  code after the colon:
  - "Certificate fulfills the technical requirements for official
    signatures": `8808`
  - "Certificate does not fulfill the technical requirements for
    official signatures": `8809`, `8810`, `8811` (depending on the
    reason)
- WS: Prevent overriding the signature manifest result in
  `VerificationReport/SignatureInfo/ManifestCheck/Manifest` with the
  XMLDsig Manifest result. Instead, put XMLDsig Manifest result in
  `VerificationReport/SignatureInfo/ManifestCheck/XMLDsigManifest`.
- Protocol Generation: Fix "PDF-AS Cant Sign PDF Reports because
  w3.org's 429 Rate Limit Error"
- Security: Bump the following third party dependencies to recent
  versions: Apache Commons Text, Apache CXF, Spring, Spring Boot,
  Xerces, XML-APIS, XmlGraphics Batik-All, XmlGraphics FOP.

### Removed

- WS: Remove the `VerificationReport/SignatureInfo/SignatureCheck`
  element upon requesting verification for a certificate.
- WS: Remove empty `soapenv:Header` Element.
- CONFIG: Ignore and remove certain elements from
  [application_config.xml](conf/signature-verification/application_config.xml)
  and make `application_config.xml` the *single source of truth* for
  configuring the verification process.
- Security: Drop needless transitive Axis dependencies.

### Upgrade Config From 2.0.5.* to 2.1.0

Pruefdienst 2.1.0 is **not backwardscompatible** with the external
configuration in `$CATALINA_BASE/conf/signature-verification` from
Pruefdienst 2.0.5.

The following steps help migrating reusable parts of an existing
Pruefdienst 2.0.5 configuration to a 2.1.0 configuration:

1. Make a backup of the existing config folder:
   ```sh
   CONF=$CATALINA_BASE/conf/signature-verification
   BACKUP=$CATALINA_BASE/conf/.signature-verification.backup.$(date +%F)
   mv $CONF $BACKUP
   ```

2. Copy the external configuration template from the Pruefdienst 2.1.0 release into `$CATALINA_BASE`:
   ```sh
   CONF210=$PATH_TO_PRUEFDIENST_2_1_0_RELEASE/conf/signature-verification
   cp -r $CONF210 $CONF
   ```

3. Inspect the new `$CONF/application_config.xml` and, if its default
   configuration is not appropriate: Manually recover values from
   `$BACKUP/application_config.xml`. **Note**: The list of parameters
   in`$CONF/application_config.xml` is exhaustive. Other parameters
   than the ones listed will be ignored by the app.

4. Optional: The configuration in subfolders `pdf-as` and `srb` in `$BACKUP/` are compatible with Pruefdienst 2.1.0 and
   can be reused as-is.
   If you want to reuse them, drop the 2.1.0 folders and replace them with the 2.0.5 folders:
   ```sh
   rm -rf $CONF/pdf-as $CONF/srb
   cp -r $BACKUP/pdf-as $BACKUP/srb $CONF
   ```

Note: Your logback configuration from Pruefdienst 2.0.5 will not work
with Pruefdienst 2.1.0 as intended because some classes have been
replaced or dropped, while other classes were moved to a new
namespace.

# Footnotes

[^1]: The new default value does not fit into the table.
    Please refer to `conf/signature-verification/application_config.xml` to view the exact new default value.
