# Signaturprüfdienst - Signature Verification Service

The signature verification service (short name: Pruefdienst) is a web
application that verifies signed documents. The app offers both a
graphical user interface and a SOAP / WSDL powered web service
interface. You can find a test deployment of this app at
<https://signaturpruefung.egiz.gv.at/test-signature-verification/>.

## Deploy Application to Tomcat

Requirements: Tomcat 10, Java >= 17

1. Copy the WAR file of the Frontend App into Tomcat's `webapps` directory:
   If run from release archive:
   
   ```
   cp pruefdienst-frontend-app.war $CATALINA_BASE/webapps/
   ```
   If run from developers repository after packaging the app:
   ```
   cp frontend-app/target/pruefdienst-frontend-app.war $CATALINA_BASE/webapps/
```
   
2. Copy the `conf/signature-verification` folder into Tomcat's `conf` directory:
   ```
   cp -r conf/signature-verification $CATALINA_BASE/conf
   ```

3. If deployed to production: Enable logback configuration in
   `conf/signature-verification/logback.xml` by removing the comment
   in `$CATALINA_BASE/conf/signature-verification/frontend-app/pruefdienst.yaml`,
   key `logging.config` and by creating a symlink to it:
   ```
   cd $CATALINA_BASE/conf/ && ln -s signature-verification/logback.xml
   ```

4. Setup [MOA SPSS](https://technology.a-sit.at/en/moa-spss/) to run
   at <http://localhost:7087/moa-spss/services/SignatureVerification>.
   In case you run MOA SPSS under another port or path: Adapt the
   following parameters to point to your MOA SPSS instance:
   - `conf/signature-verification/application_config.xml`: `moa/ss/connection.url` and `moa/sp/connection.url`
   - `conf/signature-verification/pdf-as/cfg/config.properties`: `moa.sign.url` and `moa.verify.url`

5. Start Tomcat:
   ```
   $CATALINA_BASE/bin/startup.sh
   ```
6. Find frontend GUI at `http://localhost:8080/pruefdienst-frontend-app`

## Configuration

The app can be configured externally via the `conf/signature-verification` folder.
In this folder you find:

- `conf/signature-verification/application_config.xml`: General
  configuration for legacy components. See
  `Signaturpruefservice_dokumentation.docx` Chapter 4.3.3.
  *Signaturprüfservice* for further information.

- `conf/signature-verification/formatdetection-config.xml`: Maps file
  formats to classes that can detect those file formats. See
  `Signaturpruefservice_dokumentation.docx`, Chapter 4.3.2.
  *Dokumenttyp-Erkennung* for further information.

- `conf/signature-verification/pdf-as`: PDF-AS configuration folder,
  described in `Signaturpruefservice_dokumentation.docx`, Chapter
  4.3.5 *PDF-AS*.

- `conf/signature-verification/protocolgeneration`: Contains XSLT
  stylesheet and configuration that transform verification records
  into PDF files. Not exactly intended for external configuration, but
  the underlying library wants those resources to be file system
  files, so that's how we ship them.

- `conf/signature-verification/srb`: Configuration for the detection /
  verification of srb (Strafregisterbescheinigung) documents.

- `conf/signature-verification/frontend-app/pruefdienst.yaml`:
  Contains a set of app specific parameters for the GUI and the Web Service.
  This file can also be used to configure the Spring Container.
  See inline description in the file for documentation.

- `conf/signature-verification/frontend-app/i18n`: Contains two
  `.properties` files for overriding english and german labels,
  including the RTR disclaimer at the bottom of GUI and in PDF
  verification report (key =
  `verification.result.text.overview.disclaimer.text`).

### How Internationalization (I18n) Works

When translating a key to specific languages (called
"internationalization", or i18n), the app scans a set of property
files in a specific order and translates the key with the first hit.
The following list describes the order in which the app scans the
property files (higher on the list = earlier):

1. The external configuration folder in your Tomcat deployment folder
   (`conf/signature-verification/frontend-app/i18n`)
2. A folder bundled with the web application archive (find a copy in
   `doc/i18n/frontendlabels/`)
3. A folder bundled with a dependency of the web application
   archive (find a copy in `doc/i18n/sharedlabels/`)

The sequence of these property files allows overriding labels via the
external configuration.

### How To Override I18n Labels Via Configuration

In order to override a label, you need to follow these steps:

1. Find the key that identifies the label. Consult the property files
   in `doc/i18n` to perform a "reverse lookup" of the key.

2. After you found the key, add a new entry in the property file from
   the external configuration
   `conf/signature-verification/frontend-app/i18n` with the key and
   the new label.

3. Restart the app.

## Contact

Send inquires and bug reports to
[software@egiz.gv.at](mailto:software@egiz.gv.at).
